;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
; A20 Gate
; 
; Based on osdev wiki
;-----------------------------------------------------------------------------------


; Enable the A20 gate
;--------------------

RM_Init_a20:
	call check_a20line
	cmp ax, 1
	je .ok

	call try1
	call check_a20line
	cmp ax, 1
	je .ok
	call try2
	call check_a20line
	cmp ax, 1
	je .ok
	call try3
	call check_a20line
	cmp ax, 1
	je .ok
	call try4
	call check_a20line
	cmp ax, 1
	je .ok

	jmp .noa20

.ok:
	mov ax, 240+7
	mov bx, msg_ok_a20
	mov ch,' '
	call RM_PutStr
	ret

.noa20:
	mov ax, 240+7
	mov bx, msg_no_a20
	mov ch,' '
	call RM_PutStr
	hlt				; halt cpu
	jmp $
	ret



; Enable the A20 gate 1
;----------------------
try1:
 set_A20:

	in al, 0x64
	test al, 0x02
	jnz set_A20
	mov al, 0xD1
	out 0x64, al
 check_A20:
	in al, 0x64
	test al, 0x02
	jnz check_A20
	mov al, 0xDF
	out 0x60, al

	ret
;--------------------



; Enable the A20 gate 2
;----------------------
try2:
 enable_A20:
        cli
 
        call    a20wait
        mov     al,0xAD
        out     0x64,al
 
        call    a20wait
        mov     al,0xD0
        out     0x64,al
 
        call    a20wait2
        in      al,0x60
        push    eax
 
        call    a20wait
        mov     al,0xD1
        out     0x64,al
 
        call    a20wait
        pop     eax
        or      al,2
        out     0x60,al
 
        call    a20wait
        mov     al,0xAE
        out     0x64,al
 
        call    a20wait
        sti
        ret
 
a20wait:
        in      al,0x64
        test    al,2
        jnz     a20wait
        ret
 
 
a20wait2:
        in      al,0x64
        test    al,1
        jz      a20wait2
        ret
;-----------------------


; fast a20 set 3
;---------------
try3:
 	in al, 0x92
 	or al, 2
 	out 0x92, al

	ret
;---------------------

;INT15 ax=2401 Bios 4
;--------------------
try4:
	mov ax, 2401
	int 15h

	ret
;--------------------



; OUT: 0 in ax if the a20 line is disabled (memory wraps around)
;      1 in ax if the a20 line is enabled (memory does not wrap around)
;--------------------------------------------------------------------------
check_a20line:
    pushf
    push ds
    push es
    push di
    push si
 
    cli
 
    xor ax, ax ; ax = 0
    mov es, ax
 
    not ax ; ax = 0xFFFF
    mov ds, ax
 
    mov di, 0x0500
    mov si, 0x0510
 
    mov al, byte [es:di]
    push ax
 
    mov al, byte [ds:si]
    push ax
 
    mov byte [es:di], 0x00
    mov byte [ds:si], 0xFF
 
    cmp byte [es:di], 0xFF
 
    pop ax
    mov byte [ds:si], al
 
    pop ax
    mov byte [es:di], al
 
    mov ax, 0
    je check_a20__exit
 
    mov ax, 1
 
check_a20__exit:
	
    sti

    pop si
    pop di
    pop es
    pop ds
    popf
 
    ret

; =============================================================================
; EOF
