;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
; Grab data from Infomap and set Memory Map
;-----------------------------------------------------------------------------------


base_m			dq 0
lenght_m 		dq 0
end_m			dq 0
type_m			dd 0
max_memory_address	dq 0

os_last_prg_mpages	dq 0
os_MemAmount		dd 0	; MiB
	

K_InfoMemory_map:

	; Set the address of syscalls table
	mov qword[0x0000000000005024], TABLESYSCALLS

	; Grab data from infomap
	mov rsi, 0x5000
	lodsq
	mov [os_LocalAPICAddress], rax
	lodsq
	mov [os_IOAPICAddress], rax

	mov rsi, 0x5012
	lodsw
	mov [os_NumCores], ax

	mov rsi, 0x5020
	lodsd
	mov [os_MemAmount], eax		; In MiB's

	mov rsi, 0x5031
	lodsb
	mov [os_GUI], al		; 1=ON

	mov rsi, 0x5050
	lodsq
	mov [kVBE_info_ptr], rax
	lodsq
	mov [kVBE_modeinfo_ptr], rax
	lodsw
	mov [kVBE_vesamode], ax		; current or default
	lodsb
	mov [os_IDmode], al		; ID of selected mode

	; Copy VBE tables
	mov rsi, [kVBE_info_ptr]
	mov rdi, vesa_info
	mov rcx, 64 			; 512/8
copyt1:
	lodsq
	stosq
	loop copyt1

	mov rsi, [kVBE_modeinfo_ptr]
	mov rdi, vbe
	mov rcx, 32 			; 256/8
copyt2:
	lodsq
	stosq
	loop copyt2


	; Make sure that memory range 0x300000 -> 0x3210FF is cleared
	mov rdi, os_MemoryMap
	xor rcx, rcx
	xor rax, rax
clearmem:
	stosq
	add rcx, 1
	cmp rcx, 16928			; Clear 128 KiB + 4 KiB + 256 bytes
	jne clearmem

	; Build the OS memory table
	;--------------------------
init_memory_map:

	xor rax, rax
	xor rdx, rdx

	mov rsi, 0x0000000000004000	; E820 Map location
.tablemem:
	mov rax, [rsi + 0]
	mov [base_m], rax
	mov rax, [rsi + 8]
	mov [lenght_m], rax
	mov eax, [rsi + 16]
	mov [type_m], eax
	
	cmp eax, 0			; Are we at the end?
	je .endmem

	mov rcx, [lenght_m]
	cmp rcx, 0
	je .nextmem

	mov rax, [base_m]
	mov [end_m], rax
	add [end_m], rcx
	mov rcx, [end_m]

	cmp [type_m], 1			; if usable then check the boundaries
	je .boundary
.mark:
	shr rax, 21			; base div 2 MiB : offset
	shr rcx, 21			; end  div 2 MiB : offset

	sub rcx, rax			; end-base offset
	inc rcx				; num. pages to mark

	mov rdi, os_MemoryMap
	add rdi, rax			; base offset
	mov al, byte [type_m]
	rep stosb	

.nextmem:
	add rsi, 32
	jmp .tablemem
;--------------------
.boundary:				; check if start/end are in the middle of page
	push rax			; base
	push rbx
	push rcx
	push rdx
	mov rbx, 2097152		; 2 MiB
	div rbx
	cmp rdx, 0			; remainder
	je .s0				
	mov r8, 1			; in the middle
	jmp .em
.s0:	mov r8, 0			; exactly on the bound

.em:	
	mov rax, rcx			; end
	dec rax
	mov rbx, 2097152		; 2 MiB
	div rbx
	cmp rdx, 0			; remainder
	je .e0
	mov r9, 1			; in the middle
	jmp .bdend
.e0:	mov r9, 0			; exactly on the bound
.bdend:
	pop rdx
	pop rcx
	pop rbx
	pop rax
	add rax, r8			; base
	sub rcx, r9			; end

	cmp rcx, [max_memory_address]
	jb  .mark
	mov [max_memory_address], rcx
	jmp .mark
;-----------------
.endmem:

	mov rdi, os_MemoryMap
	mov al, 2
	stosb				; Mark the first 2 MiB as in use (by SYS64 and PD)
	stosb				; As well as the second 2 MiB (by AP's stacks and Mmap)
	stosb				; and the third 2 Mib (by kernel)
					; The CLI should take care of the program memory
ret



;------------------------
; IN : rcx : size (bytes)
K_mmap_prg_add:
	push rax
	push rcx
	push rdi

	shr rcx, 21			; div 2 MiB
	add rcx, 1			; num. pages to allocate
	mov [os_last_prg_mpages], rcx
	mov rdi, os_MemoryMap
	add rdi, 3			; skip kernel
	mov al, 2
	rep stosb

	pop rdi
	pop rcx
	pop rax
	ret

;-------------------
K_mmap_prg_release:
	push rax
	push rcx
	push rdi

	mov rcx, [os_last_prg_mpages]
	mov rdi, os_MemoryMap
	add rdi, 3			; skip kernel
	mov al, 1
	rep stosb

	pop rdi
	pop rcx
	pop rax
	ret


;-----------------------------------------------------------------------------------------------
; Memory example:
;
; E820 table
; 
;  00000000 - 0009f800 - 01
;  000f0000 - 00010000 - 02
;  fec00000 - 01400000 - 02
;  c0000000 - 10000000 - 02
;  0009f800 - 00000800 - 02
;  bdcf0000 - 00010000 - 02
;  00100000 - bdbe0000 - 01
;  bdce3000 - 0000d000 - 03
;  bdce0000 - 00003000 - 04
; 100000000 - 40000000 - 01
;
; Base      - Lenght   - Type
;
; A part of ram, in PC there are graphic cards, hard disk and so on.
; Historically, the PCI bus in PC has shared the same physical address space as the CPU.
; So, in the original 8086, 1 MiB is split in 640 KiB for memory and 384 KiB for other (devices).
; The same problem exists today with 32 bit addressing and 4 GiB of ram. Typically the bottom
; 3 GiB is for memory, and the top 1 GiB for devices: like the old DOS time, there are 'holes'.
; The usual solution for using all 4 GiB is to make that last GiB appear further up in the memory map.
; So your memory is all still available, it just happens not to be contiguous.
; The first 3 GiB are present where you'd expect to find them, then there is a 1 GiB hole, reserved for devices,
; and the 4th GiB of memory appears in the 5th GiB of address space.


