;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
;
; Init Graph Mode Routines
;-----------------------------------------------------------------------------------


;---------------------------------------------------------------
; Convert seg:off address to linear address (32 bit)
; IN : EAX = seg:off (off:0-15 seg:16-31)
; OUT: EAX = linear address
segoff_to_linear:
	push rbx
	mov ebx, eax
	shr eax, 12
	and eax, 0xFFFF0
	and ebx, 0xFFFF
	add eax, ebx
	pop rbx
ret


;---------------------------------------------------------------
; Auxiliary function for printing ModeInfoBlock data
; IN :  RAX = number
;	RBX = string
print_ns:
	push rsi
	push rdi

	mov rsi, rbx
	call os_print_string

	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string
	mov rsi, space
	call os_print_string

	pop rdi
	pop rsi
ret


;---------------------------------------------------------------
; Set number of columns/rows in text mode and graph mode
set_colsrows_limits:

	cmp [os_GUI], 0
	je .text
.graph:
	mov byte[ncols], 160
	mov byte[nrows], 60
	mov byte[nrowsdir], 55
	mov byte[startcols], 0
	mov byte[startrows], 0
	jmp .end
.text:
	mov byte[ncols], 80
	mov byte[nrows], 25
	mov byte[nrowsdir], 20
	mov byte[startcols], 0
	mov byte[startrows], 0
.end:
ret


;---------------------------------------------------------------
; Set different chars between different codepages in text mode
; (437) and graph mode (ISO)
; e.g.: in codepage 437 the degree is 248, in ISO-8859-15 is 176
; so, i change the code in keymap
;---------------------------------------------------------------
set_kb_codepage:			; (graph mode only)
	push rdi

	; italian keyboard
	mov rdi, normal_keymap_IT
	mov byte[rdi+13], 236
	mov byte[rdi+26], 232
	mov byte[rdi+39], 242
	mov byte[rdi+40], 224
	mov byte[rdi+43], 249
	mov rdi, shift_keymap_IT	
	mov byte[rdi+4],  163
	mov byte[rdi+26], 233	
	mov byte[rdi+39], 231
	mov byte[rdi+40], 176
	mov byte[rdi+43], 167
	mov rdi, caplock_keymap_IT
	mov byte[rdi+13], 236
	mov byte[rdi+26], 232
	mov byte[rdi+39], 242
	mov byte[rdi+40], 224
	mov byte[rdi+43], 249

	pop rdi
ret


;---------------------------------------------------------------
; Set ISO-8859-15 codepage and system font in graph mode
set_system_font:

	cmp [os_GUI], 0			; Graph mode only
	je .end


	; set the appropriate codes for
	; ISO-8859-15 standard
	;-------------------------------
	call set_kb_codepage


	; set the appropriate font for
	; screen resolution
	;-------------------------------
	cmp [os_IDmode], 1
	je .ID1
	cmp [os_IDmode], 2
	je .ID2
	cmp [os_IDmode], 3
	je .ID3
	cmp [os_IDmode], 4
	je .ID4
	cmp [os_IDmode], 5
	je .ID5
	cmp [os_IDmode], 6
	je .ID6
	cmp [os_IDmode], 7
	je .ID7
	cmp [os_IDmode], 8
	je .ID8
	cmp [os_IDmode], 9
	je .ID9
	cmp [os_IDmode], 10
	je .ID10

	.ID1:					; 1280x720
	mov [os_System_Font], font8x12
	mov [os_System_Font_xsize],   8
	mov [os_System_Font_ysize],  12
	mov [os_System_Font_xysize], 96
	mov [os_Disp_x], 0
	mov [os_Disp_y], 0
	jmp .end	
	.ID2:					; 1600x900
	mov [os_System_Font], font10x15
	mov [os_System_Font_xsize],  10
	mov [os_System_Font_ysize],  15
	mov [os_System_Font_xysize], 150
	mov [os_Disp_x], 0
	mov [os_Disp_y], 0
	jmp .end
	.ID3:					; 1920x1080
	mov [os_System_Font], font12x18
	mov [os_System_Font_xsize],  12
	mov [os_System_Font_ysize],  18
	mov [os_System_Font_xysize], 216
	mov [os_Disp_x], 0
	mov [os_Disp_y], 0
	jmp .end
	.ID4:					; 2560x1440
	mov [os_System_Font], font16x24
	mov [os_System_Font_xsize],  16
	mov [os_System_Font_ysize],  24
	mov [os_System_Font_xysize], 384
	mov [os_Disp_x], 0
	mov [os_Disp_y], 0
	jmp .end
	.ID5:					; 1280x800
	mov [os_System_Font], font8x12
	mov [os_System_Font_xsize],   8
	mov [os_System_Font_ysize],  12
	mov [os_System_Font_xysize], 96
	mov [os_Disp_x], 0
	mov [os_Disp_y], 40
	jmp .end
	.ID6:					; 1680x1050
	mov [os_System_Font], font10x15
	mov [os_System_Font_xsize],  10
	mov [os_System_Font_ysize],  15
	mov [os_System_Font_xysize], 150
	mov [os_Disp_x], 40
	mov [os_Disp_y], 75
	jmp .end
	.ID7:					; 1920x1200
	mov [os_System_Font], font12x18
	mov [os_System_Font_xsize],  12
	mov [os_System_Font_ysize],  18
	mov [os_System_Font_xysize], 216
	mov [os_Disp_x], 0
	mov [os_Disp_y], 60
	jmp .end
	.ID8:					; 2560x1600
	mov [os_System_Font], font16x24
	mov [os_System_Font_xsize],  16
	mov [os_System_Font_ysize],  24
	mov [os_System_Font_xysize], 384
	mov [os_Disp_x], 0
	mov [os_Disp_y], 80
	jmp .end
	.ID9:					; 800x600
	mov [os_System_Font], font5x10
	mov [os_System_Font_xsize],   5
	mov [os_System_Font_ysize],  10
	mov [os_System_Font_xysize], 50
	mov [os_Disp_x], 0
	mov [os_Disp_y], 0
	jmp .end
	.ID10:					; 1024x768
	mov [os_System_Font], font6x12
	mov [os_System_Font_xsize],   6
	mov [os_System_Font_ysize],  12
	mov [os_System_Font_xysize], 72
	mov [os_Disp_x], 32
	mov [os_Disp_y], 24
	;jmp .end

.end:
ret


;-------------------------------------------------------------------
; Convert color format from graph mode to text mode (only 16 colors)
; IN : RBX = color (graph mode)
; OUT: RBX = color (text mode)
set_color_text:
	push rax
	push rcx

	xor rax, rax
	mov rcx, 1

	ror rbx, 32		; background
.color:
	cmp ebx, ct_black
	je .black
	cmp ebx, ct_blue
	je .blue
	cmp ebx, ct_green
	je .green
	cmp ebx, ct_cyan
	je .cyan
	cmp ebx, ct_red
	je .red
	cmp ebx, ct_magenta
	je .magenta
	cmp ebx, ct_brown
	je .brown
	cmp ebx, ct_lgrey
	je .lgrey
	cmp ebx, ct_dgrey
	je .dgrey
	cmp ebx, ct_lblue
	je .lblue
	cmp ebx, ct_lgreen
	je .lgreen
	cmp ebx, ct_lcyan
	je .lcyan
	cmp ebx, ct_lred
	je .lred
	cmp ebx, ct_lmagenta
	je .lmagenta
	cmp ebx, ct_yellow
	je .yellow
	cmp ebx, ct_white
	je .white

	.black:
	or al, 00000000b
	jmp .next
	.blue:
	or al, 00000001b
	jmp .next
	.green:
	or al, 00000010b
	jmp .next
	.cyan:
	or al, 00000011b
	jmp .next
	.red:
	or al, 00000100b
	jmp .next
	.magenta:
	or al, 00000101b
	jmp .next
	.brown:
	or al, 00000110b
	.lgrey:
	or al, 00000111b
	jmp .next
	.dgrey:
	or al, 00001000b
	jmp .next
	.lblue:
	or al, 00001001b
	jmp .next
	.lgreen:
	or al, 00001010b
	jmp .next
	.lcyan:
	or al, 00001011b
	jmp .next
	.lred:
	or al, 00001100b
	jmp .next
	.lmagenta:
	or al, 00001101b
	jmp .next
	.yellow:
	or al, 00001110b
	.white:
	or al, 00001111b
	;jmp .next

.next:
	cmp rcx, 2
	je .end

	inc rcx
	rol rbx, 32		; foreground
	rol rax, 4		;

	jmp .color
.end:
	mov rbx, rax		; return color (text mode)

	pop rcx
	pop rax
	ret


; ===================================================================================================
; EOF


; 160x60
;
;	16/9			    16/10			4/3 (Not Av.)
;       ---------------------------------------------------------------------------------------------------
;	Av. Resol.    Font  Disp.   Av. Resol.    Font  Disp.   Av. Resol.    Font  Disp.	Size (In.)
;	---------------------------------------------------------------------------------------------------
;	*   1280x720   8x12  0-0    *	1280x800   8x12	 0-80	    1280x960   8x16  0-0	15'-17'
;	*   1600x900  10x15  0-0    *	1680x1050 10x15 80-150	    1600x1200 10x20  0-0	20'
;	*   1920x1080 12x18  0-0    *	1920x1200 12x18  0-120	    1920x1440 12x24  0-0	22'-24'
;	*   2560x1440 16x24  0-0    *	2560x1600 16x24  0-160	    2560x1920 16x32  0-0	27'
;
;							        *    800x600   5x10  0-0  ; default and
;							        *   1024x768   6x12 64-48 ; deprecated resolutions


