;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
; System Variables
;-----------------------------------------------------------------------------------


; Memory addresses
;-----------------
buffer64		equ 0x0000000000010000	; 65536 bytes 0x10000 -> 0x1FFFF - 64 KiB Buffers in low memory
os_MemoryMap		equ 0x0000000000300000  ; 131072 bytes	0x300000 -> 0x31FFFF - Location of Memory Map - Room to map 256 GiB with 2 MiB pages
cpustatus		equ 0x0000000000320000	; 256 bytes  0x320000 -> 0x3200FF - Location of CPU status data  Bit 0 = Available, Bit 1 = Free/Busy
cpuqueue		equ 0x0000000000320100	; 4096 bytes 0x320100 -> 0x3210FF - Location of CPU Queue. Each queue item is 16 bytes. (Room for 256 entries)

ata0_header_cache	equ 0x0000000000360000  ; 4096 bytes 0x360000 -> 0x360FFF - Location of header cache    ata0
ata0_root_cache		equ 0x0000000000361000	; 4096 bytes 0x361000 -> 0x361FFF - Location of root cache	ata0
ata1_header_cache	equ 0x0000000000362000  ; 4096 bytes 0x362000 -> 0x362FFF - Location of header cache    ata1
ata1_root_cache		equ 0x0000000000363000	; 4096 bytes 0x363000 -> 0x363FFF - Location of root cache	ata1
ata2_header_cache	equ 0x0000000000364000  ; 4096 bytes 0x364000 -> 0x364FFF - Location of header cache    ata2
ata2_root_cache		equ 0x0000000000365000	; 4096 bytes 0x365000 -> 0x365FFF - Location of root cache	ata2
ata3_header_cache	equ 0x0000000000366000  ; 4096 bytes 0x366000 -> 0x366FFF - Location of header cache    ata3
ata3_root_cache		equ 0x0000000000367000	; 4096 bytes 0x367000 -> 0x367FFF - Location of root cache	ata3

						; 0x368000 -> 0x3EFFFF Reserved for future devices

header_buffer		equ 0x00000000003F0000  ; 4096 bytes 0x3F0000 -> 0x3F0FFF - Location of header buffer
root_buffer		equ 0x00000000003F1000  ; 4096 bytes 0x3F1000 -> 0x3F1FFF - Location of root buffer
cluster_buffer		equ 0x00000000003F2000  ; 4096 bytes 0x3F2000 -> 0x3F2FFF - Location of cluster buffer
cluster_buffer2		equ 0x00000000003F3000  ; 4096 bytes 0x3F3000 -> 0x3F3FFF - Location of cluster buffer2

						; 0x400000 Kernel
programlocation		equ 0x0000000000600000	; Location in memory where programs are loaded


; Variables
;----------

align 16
temp_str: times 21 	db 0			; 20 digit + 1 string terminator
align 16
str_8digit 		db '00000000', 0
align 16
hextable: 		db '0123456789ABCDEF'
align 16
EOF			db  26 ; ascii

align 16
cli_args		db 0
align 16
prg_args		db 0
align 16
cli_command_args_string:	times 1024 db 0
align 16
prg_path_args_string:		times 1024 db 0
align 16
cli_key_com:			times 10240 db 0
align 16
cli_key_com_counter 		db 0

align 16
directory_cache_ptrs:		times 288 db 0	; 24*12 => 64bit=device, 64bit=lba, 64bit=memory (12 pages * 2MiB)

align 16
sector0: 		times 512 db 0
align 16
sector1: 		times 512 db 0
align 16
sector2: 		times 512 db 0
align 16
device_list 		db 'Devices list: '	; Identified devices list
device_list_room:	times 256 db 0		; (don't move it)

uuid1 			dq 0	; fs_format.inc
uuid2 			dq 0	; 

os_LocalAPICAddress	dq 0
os_IOAPICAddress	dq 0
os_ClockCounter		dq 0
os_RandomSeed		dq 0	; Seed for RNG
os_Ticks		dq 0
screen_cursor_offset	dq 0

mutex 			dw 1	; The MUTual-EXclustion flag (in cli.inc)
os_NumCores		dw 0
cpuqueuestart		dw 0
cpuqueuefinish		dw 0
os_QueueLen		dw 0
os_QueueLock		dw 0	; Bit 0 clear for unlocked, set for locked.
cpuqueuemax:		dw 256

cursorx			db 0
cursory			db 0
scancode		db 0
key			db 0
key_shift		db 0
screen_cursor_x		db 0
screen_cursor_y		db 0
os_debug_dump_reg_stage	db 0
hd_ide_mask		db 0
os_GUI			db 0
os_IDmode		db 0

align 16
colsrows_struct:		; (don't move it)
nrows			db 0	;
ncols			db 0	;
startrows		db 0	;
startcols		db 0	;
nrowsdir		db 0

os_System_Font		dq 0
os_System_Font_xsize	dq 0
os_System_Font_ysize	dq 0
os_System_Font_xysize	dq 0
os_Disp_x		dq 0
os_Disp_y		dq 0
os_Color_bgfg		dq 0

kVBE_info_ptr 		dq 0
kVBE_modeinfo_ptr 	dq 0
kVBE_vesamode		dw 0

;---------------------------
; Tiles and Controls
;---------------------------

current_tile		dq 0
tileslist   		dq 0
current_ctrl    	dq 0
ctrlslist		dq 0
exitUI 			db 0

os_Color_focus_off	dq 0x0000ff0000000000	; ct_black_on_green
os_Color_focus_on 	dq 0x00ff000000000000	; ct_black_on_red

; colors
ct_white_on_dgrey equ 0x0066666600ffffff ; text
ct_black_on_white equ 0x00ffffff00000000 ; tile

ct_black_on_lgrey equ 0x00cccccc00000000 ; text (items)
ct_white_on_lgrey equ 0x00cccccc00ffffff ; ctrl
ct_white_on_black equ 0x0000000000ffffff ; input (items)

; type
;-not used-	equ 0
TILE		equ 1
CHECK 		equ 2
LABEL		equ 3	; always disable
ACTIVELABEL	equ 4	; tag=1 -> actived
ARRAYALABEL	equ 5	; tag=id actived label
INPUTSTRING	equ 6
BITMAP		equ 7
CUSTOMCONTROL   equ 8

; tile/control offset
TYPE	 	equ 0
FLAGS	 	equ 1
IDCHAR 		equ 2
FOCUS	 	equ 3
YPOS 		equ 4
XPOS 		equ 5
YSIZE	 	equ 6
XSIZE	 	equ 7
SELECTED 	equ 8
NITEMS	 	equ 16
NUMBER	 	equ 24
TAG 		equ 32
TEXT	 	equ 40
PTPOINTER 	equ 48
COUNT		equ 56
COLOR1		equ 64
COLOR2		equ 72
COLOR3		equ 80
MLIST		equ 88
VALUE		equ 96
FUNCTION	equ 104
DRAW		equ 112
ACTION		equ 120

; flags
VD		equ 0	; visible & disable
VE		equ 1	; visible & enable
HD		equ 2	; hidden & disable
TH		equ 3	; to hide

;------------------------------------------------------------------------


;-------------------------------------
; Color text attribute
;-------------------------------------
;			back	fore
;			argb    argb		text mode (b f)
;		      <------><------>	
ct_black	equ 0x0000000000000000	;	0x00 (0000 0000b)
ct_blue		equ 0x00000000000000ff	;	0x01 (0000 0001b)
ct_green	equ 0x000000000000ff00	;	0x02 (0000 0010b)
ct_cyan		equ 0x000000000000ffff	;	0x03 (0000 0011b)
ct_red      	equ 0x0000000000ff0000	;	0x04 (0000 0100b)
ct_magenta  	equ 0x0000000000ff00ff	;	0x05 (0000 0101b)
ct_brown   	equ 0x0000000000996600	;	0x06 (0000 0110b)
ct_lgrey   	equ 0x0000000000999999	;	0x07 (0000 0111b)

ct_dgrey   	equ 0x0000000000666666	;	0x08 (0000 1000b)
ct_lblue   	equ 0x00000000006666ff	;	0x09 (0000 1001b)
ct_lgreen  	equ 0x000000000066ff66	;	0x0a (0000 1010b)
ct_lcyan    	equ 0x000000000066ffff	;	0x0b (0000 1011b)
ct_lred     	equ 0x0000000000ff6666	;	0x0c (0000 1100b)
ct_lmagenta 	equ 0x0000000000ff66ff	;	0x0d (0000 1101b)
ct_yellow   	equ 0x0000000000ffff00	;	0x0e (0000 1110b)
ct_white    	equ 0x0000000000ffffff	;	0x0f (0000 1111b)
;		    < 63-32 > < 31-0 >
; lbrown=yellow


;--------------------------------------------------------------
;Bitfields for character's display attribute:       (text mode)
;Bit(s)	Description
; 7	foreground blink or (alternate) background bright
; 6-4	background color 
; 3	foreground bright or (alternate) alternate character set
; 2-0	foreground color 
;---------------------------------------------------------------

; ===================================================================================================
; EOF

