;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
;
; Tiles and Controls: Draw and Manager
;-----------------------------------------------------------------------------------

;os_init_UI
;os_exit_UI
;os_tilectrl_set_focus_colors
;os_tile_draw_message
;os_tile_draw
;os_ctrl_draw


; -----------------------------------------------------------------------------
; os_init_UI -- Initialize User Interface manager with tiles and controls 
; IN :  RAX = tiles_list
;	RBX = tile_a_pages
; OUT:	All registers preserved
os_init_UI:

	call tile_ctrl_init		; save pointers and set first tile as current

	call tile_draw_all		; draw all tiles (and controls)

	call Tmanager			; *main*

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_exit_UI -- Exit from User Interface manager
; IN :	none
; OUT:	none
os_exit_UI:

	mov byte[exitUI], 1

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; tile_ctrl_init -- Save pointers and set first tile as current
; IN :	RAX = tiles list pointer
; 	RBX = pages pointer (first tile)
; OUT:	All registers preserved
tile_ctrl_init:
	push rax
	push rbx
	push rcx

	mov [tileslist], rax		; tiles list pointer
	add rax, 8
	mov rcx, [rax]
	mov [current_tile], rcx		; first tile pointer

	mov [ctrlslist], rbx		; controls list pointer
	mov qword[current_ctrl], 0	; control pointer

	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; tile_draw_all -- Draw all tiles (and controls)
; IN :  RAX = tiles list pointer	 [tileslist]
;	RBX = pages pointer (first tile) [ctrlslist]
; OUT:	All registers preserved
tile_draw_all:
	push rax
	push rbx
	push rcx

	mov rcx, [rax]			; number of tiles
	mov rbx, rax 			; tiles_list
	add rbx, 8			; now points to tile
	mov rax, [rbx]			; pointer
.next:
	call os_tile_draw		; draw
	add rbx, 8			; next tile
	mov rax, [rbx]			; pointer
	loop .next
.end:
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; Tmanager -- The main cicle, it manages the UI
; IN :	none
; OUT:	All registers preserved
Tmanager:
	push rax
	push rbx
	push rcx
	push rdx

	mov byte[exitUI], 0

.cicle:
	cmp byte[exitUI], 1
	je .exit

	call os_input_key_wait
	call os_is_lowercase
	je .lower
	call os_is_uppercase
	je .up
	call os_is_ctrl_letter
	je .tile

	cmp ah, 73			; If PgUp key pressed, next page
	je .nextpage
	cmp ah, 81			; If PgDw key pressed, prev page
	je .prevpage

	jmp .cicle


;--------------------------------------- controls
.lower:
	xor rbx, rbx
	mov bl, al
	sub bl, 97			; 'a'
	mov rax, [ctrlslist]		; list of controls
	;----
	mov rcx, [rax]			; check number of controls
	dec rcx 
	cmp rbx, rcx			
	ja .cicle
	;----
	shl rbx, 3			; mul 8
	add rax, 8			; now points to first ctrl pointer
	add rbx, rax			; add offset
	mov rax, [rbx]			; ctrl pointer
	call ctrl_set_focus_on
	call ctrl_action
	call ctrl_function
	call ctrl_set_focus_off

	jmp .cicle

.up:
	xor rbx, rbx
	mov bl, al
	sub bl, 65			; 'A'
	add rbx, 26			; add lower
	mov rax, [ctrlslist]		; list of controls
	;----
	mov rcx, [rax]			; check number of controls
	dec rcx 
	cmp rbx, rcx			
	ja .cicle
	;----
	shl rbx, 3			; mul 8
	add rax, 8			; now points to first ctrl pointer
	add rbx, rax			; add offset
	mov rax, [rbx]			; ctrl pointer
	call ctrl_set_focus_on
	call ctrl_action
	call ctrl_function
	call ctrl_set_focus_off

	jmp .cicle


;--------------------------------------- tiles	
.tile:
	xor rbx, rbx
	mov bl, al			; 1-26
	dec bl
	mov rax, [tileslist]		; list of tiles
	;----
	mov rcx, [rax]			; check number of tiles
	dec rcx
	cmp rbx, rcx			
	ja .cicle
	;----
	shl rbx, 3			; mul 8
	add rax, 8			; now points to first tile pointer
	add rbx, rax			; add offset
	mov rax, [rbx]			; tile pointer
	call tile_set_focus
	jmp .cicle

;--------------------------------------- next
.nextpage:
	mov rax, [current_tile]
	mov rbx, [rax+PTPOINTER]	; pages list pointer

	mov rdx, [rax+SELECTED]		; selected page
	cmp rdx, [rax+NITEMS]
	je .cicle

	inc rdx				; next
	mov [rax+SELECTED], rdx		; store value
	call _tile_set_page_pointer
	mov [ctrlslist], rbx		; store value

	call os_tile_draw
	jmp .cicle

;--------------------------------------- prev
.prevpage:
	mov rax, [current_tile]
	mov rbx, [rax+PTPOINTER]	; pages list pointer

	mov rdx, [rax+SELECTED]		; selected page
	cmp rdx, 1
	je .cicle

	dec rdx				; prev
	mov [rax+SELECTED], rdx		; store value
	call _tile_set_page_pointer
	mov [ctrlslist], rbx		; store value

	call os_tile_draw
	jmp .cicle

;---------------------------------------
.exit:
	call os_screen_clear
	mov ax, 0x0001
	call os_move_cursor

	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------



;-----------------------------------------------------
;	Tiles Functions
;-----------------------------------------------------

; -----------------------------------------------------------------------------
; os_tilectrl_set_focus_colors -- Set the colors of on/off focus
; IN :  RAX = color of focus on
;	RBX = color of focus off
; OUT:	All registers preserved
os_tilectrl_set_focus_colors:

	mov [os_Color_focus_on] , rax
	mov [os_Color_focus_off], rbx
ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_tile_draw_message -- Draw the message of tile
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
os_tile_draw_message:
	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	call _tile_draw_firstline
	call _tile_draw_msgtext
	call _tile_draw_npage
	call _tilectrl_draw_focus
.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_tile_draw -- Draw the tile (and the controls)
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
os_tile_draw:
	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	call _tile_draw_panel

	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	call _tile_draw_firstline
	call _tile_draw_msgtext
	call _tile_draw_npage
	call _tilectrl_draw_focus
	call _tile_draw_ctrls
.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; tile_set_focus -- Set focus of current(off) and new(on) focused tiles
; IN :	RAX = pointer to new focused tile
; OUT:	All registers preserved
tile_set_focus:

	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	cmp byte[rax+FLAGS], VD		; visible & disable
	je .end

	push rax
	mov rax, [current_tile]
	mov byte[rax+FOCUS], 0		; focus off (current)
	call _tilectrl_draw_focus
	pop rax

	mov byte[rax+FOCUS], 1		; focus on (new)
	call _tilectrl_draw_focus

	push rbx
	push rdx
	mov rbx, [rax+PTPOINTER]	; pages list pointer
	mov rdx, [rax+SELECTED]		; selected page
	call _tile_set_page_pointer
	mov [ctrlslist], rbx		; store value	
	pop rdx
	pop rbx

	mov [current_tile], rax		; store value
.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_draw_panel -- Draw the panel of tile, the fist line is skipped
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
_tile_draw_panel:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rdx, rax
	mov rbx, qword[rdx+COLOR2] 	; *color tile

	; draw panel
	;------------------
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	mov cl, byte[rdx+YSIZE] 	; ysize
	add cl, al
	movzx rsi, cl	    		; y end
	inc al				; skip the first line

	cmp byte[rdx+FLAGS], TH		; to hide
	jne .nohidden
	mov rbx, [os_Color_bgfg]
	mov byte[rdx+FLAGS], HD
	dec al				; include the first line
	.nohidden:

	xor rcx, rcx
.more:
	call os_move_cursor
	mov cl, byte[rdx+XSIZE] 	; xsize

	push rax
	mov al, ' '

	cmp byte[rdx+FLAGS], VD		; visible & disable
	jne .draw
	mov al, 'X'

.draw:
	call os_print_char_with_color
	loop .draw
	pop rax

	inc al				; next line
	movzx rdi, al
	cmp rsi, rdi
	jne .more
.end:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_draw_firstline -- Draw the first line of tile (empty)
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
_tile_draw_firstline:
	push rax
	push rbx
	push rcx
	push rdx

	mov rdx, rax
	mov rbx, qword[rdx+COLOR1] 	; *color msg

	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	call os_move_cursor
	xor rcx, rcx
	mov cl, byte[rdx+XSIZE] 	; xsize
	mov al, ' '
.draw:
	call os_print_char_with_color
	loop .draw

	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tilectrl_draw_focus -- Draw the focus (both tile and control)
; IN :	RAX = pointer to tile or control struct
; OUT:	All registers preserved
_tilectrl_draw_focus:
	push rax
	push rbx
	push rcx
	push rdx

	mov rdx, rax
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	call os_move_cursor

	mov rbx, qword[os_Color_focus_off] ; *color char id focus off
	mov cl, byte[rdx+FOCUS]	   	   ; focus
	cmp cl, 0			   ; if off jump to print
	je .print
	mov rbx, qword[os_Color_focus_on]  ; *color char id focus on
.print:
	mov al, byte[rdx+IDCHAR]	; focus char id
	call os_print_char_with_color

	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_draw_msgtext -- Draw the text message
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
_tile_draw_msgtext:
	push rax
	push rbx
	push rdx
	push rsi

	mov rdx, rax
	mov rbx, qword[rdx+COLOR1] 	; *color msg
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS]	 	; x
	add ah, 2
	call os_move_cursor
	mov rsi, qword[rdx+TEXT] 	; text string pointer
	call os_print_string_with_color

	pop rsi
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_draw_npage -- Draw the number of selected page / total pages
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
_tile_draw_npage:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rdx, rax
	mov rbx, qword[rdx+COLOR1] 	; *color msg
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	mov cl, byte[rdx+XSIZE] 	; xsize
	add ah, cl

	mov cl, 3
	mov ch, byte[rdx+NITEMS] 	; num. pages (max 99)
	cmp ch, 9
	jbe .move
	mov cl, 5
	;---
	mov ch, byte[rdx+SELECTED] 	; selected page (max 99)
	cmp ch, 9
	ja .move
	dec cl

.move:
	sub ah, cl
	call os_move_cursor

	xor rax, rax
	mov al, byte[rdx+SELECTED] 	; selected page
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string_with_color

	mov al, '/'
	call os_print_char_with_color

	xor rax, rax
	mov al, byte[rdx+NITEMS] 	; num. pages	
	mov rdi, temp_str
	call os_int_to_string
	mov rsi, temp_str
	call os_print_string_with_color

	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_set_page_pointer -- Set in rbx the pointer to the selected page
; IN :	RBX = pointer to pages list
; 	RDX = selected page
; OUT:	RBX = pointer to pages list
_tile_set_page_pointer:
	push rcx
	push rdx

	dec rdx				; num. offset
	cmp rdx, 0
	je .pageok
.again:
	mov rcx, [rbx]			; number of ctrls	
	inc rcx
.nextpage:
	add rbx, 8
	loop .nextpage
	dec rdx
	cmp rdx, 0
	je .pageok
	jmp .again
.pageok:
	pop rdx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _tile_draw_ctrls -- Draw the controls of the tile
; IN :	RAX = pointer to tile struct
; OUT:	All registers preserved
_tile_draw_ctrls:
	push rax
	push rbx
	push rcx
	push rdx

	mov rbx, [rax+PTPOINTER]	; pointer to pages list
	mov rdx, [rax+SELECTED]		; selected page

	call _tile_set_page_pointer	

	mov rcx, [rbx]			; number of ctrls
	add rbx, 8			; now points to first ctrl
	mov rax, [rbx]			; pointer
	cmp rcx, 0
	je .end
.next:
	call os_ctrl_draw
	add rbx, 8			; next ctrl
	mov rax, [rbx]			; pointer
	loop .next
.end:
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------




;-----------------------------------------------------
;	Controls Functions
;-----------------------------------------------------

; -----------------------------------------------------------------------------
; ctrl_set_focus_on -- Set the focus of control ON
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
ctrl_set_focus_on:
	push rbx

	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	cmp byte[rax+FLAGS], VD		; visible & disable
	je .end

	mov rbx, [rax+PTPOINTER]
	cmp byte[rbx+FLAGS], VD		; tile visible & disable
	je .end

	mov byte[rax+FOCUS], 1		; focus on
	call _tilectrl_draw_focus	
	mov [current_ctrl], rax

.end:
	pop rbx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; ctrl_set_focus_off -- Set the focus of control OFF
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
ctrl_set_focus_off:
	push rbx

	cmp byte[rax+FLAGS], HD		; hidden & disable
	je .end

	cmp byte[rax+FLAGS], VD		; visible & disable
	je .end

	mov rbx, [rax+PTPOINTER]
	cmp byte[rbx+FLAGS], VD		; tile visible & disable
	je .end

	mov byte[rax+FOCUS], 0		; focus off
	call _tilectrl_draw_focus	
	mov qword[current_ctrl], 0

.end:
	pop rbx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _ctrl_draw_firstline -- Draw the first line of control (empty)
; IN :	RDX = pointer to control struct
; OUT:	All registers preserved
_ctrl_draw_firstline:
	push rax
	push rbx
	push rcx
	push rsi	

	cmp byte[rdx+TYPE], LABEL	; (always disable, no focus)
	je .text

	mov al, ' '
	mov rbx, qword[rdx+COLOR1] 	; *color text
	call os_print_char_with_color
.text:
	mov rbx, qword[rdx+COLOR1] 	; *color text
	mov rsi, qword[rdx+TEXT] 	; text string pointer
	call os_print_string_with_color

	; special cases
	mov rbx, qword[rdx+COLOR3] 	; *color items
	cmp byte[rdx+TYPE], INPUTSTRING
	je .pad
	cmp byte[rdx+TYPE], BITMAP
	je .pad
	; normal
	mov rbx, qword[rdx+COLOR1] 	; *color text
.pad:
	mov al, ' '			; pad the control with space
	mov ch, byte[rdx+XPOS] 		; x start
	add ch, byte[rdx+XSIZE]		; x end
.while:
	mov ah, [screen_cursor_x]
	cmp ah, ch
	je .continue
	call os_print_char_with_color
	jmp .while
	.continue:

	pop rsi
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; _ctrl_tohidedisable -- Hide or disable the control
; IN :	RDX = pointer to control struct
; OUT:	All registers preserved
_ctrl_tohidedisable:
	push rax
	push rbx
	push rcx

	cmp byte[rdx+FLAGS], TH		; to hide
	je .tohide
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .disable
	jmp .end			; otherwise (enable)

.disable:
	push rdx
	mov rbx, qword[rdx+COLOR2] 	; *color ctrl
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	mov cl, byte[rdx+XSIZE]		; x sixe
	mov ch, byte[rdx+YSIZE]		; y size
	mov dl, 'X'			; !!!
	call os_print_box_with_color
	pop rdx

	mov rax, rdx
	call _tilectrl_draw_focus	; draw focus
	jmp .end

.tohide:
	push rdx
	mov byte[rdx+FLAGS], HD		; hidden & disable
	mov rax, [rdx+PTPOINTER]	; pointer to tile	
	mov rbx, qword[rax+COLOR2]	; *color tile
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	mov cl, byte[rdx+XSIZE]		; x sixe
	mov ch, byte[rdx+YSIZE]		; y sixe
	mov dl, ' '			; !!!
	call os_print_box_with_color
	pop rdx
	;jmp .end

.end:
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_ctrl_draw -- Draw the control (switch)
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
os_ctrl_draw:

	cmp byte[rax+FLAGS], HD		; ctrl hidden & disable
	je .end

	cmp byte[rax+TYPE], CHECK
	je .check
	cmp byte[rax+TYPE], LABEL
	je .label
	cmp byte[rax+TYPE], ACTIVELABEL
	je .activelabel
	cmp byte[rax+TYPE], ARRAYALABEL
	je .arrayalabel
	cmp byte[rax+TYPE], INPUTSTRING
	je .inputstring
	cmp byte[rax+TYPE], BITMAP
	je .bitmap
	cmp byte[rax+TYPE], CUSTOMCONTROL
	je .customcontrol

	jmp .end

;---------------------------------------
;	CHECK
;---------------------------------------
.check:
	call check_draw
	jmp .end
;---------------------------------------
;	LABEL
;---------------------------------------
.label:
	call label_draw
	jmp .end
;---------------------------------------
;	ACTIVELABEL
;---------------------------------------
.activelabel:
	call activelabel_draw
	jmp .end
;---------------------------------------
;	ARRAYALABEL
;---------------------------------------
.arrayalabel:
	call arrayalabel_draw
	jmp .end
;---------------------------------------
;	INPUTSTRING
;---------------------------------------
.inputstring:
	call inputstring_draw
	jmp .end
;---------------------------------------
;	BITMAP
;---------------------------------------
.bitmap:
	call bitmap_draw
	jmp .end
;---------------------------------------
;	CUSTOMCONTROL
;---------------------------------------
.customcontrol:
	push rbx
	mov rbx, qword[rdx+DRAW] 	; pointer to draw user function
	call rbx
	pop rbx
	jmp .end

.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; ctrl_action -- The control in action (switch)
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
ctrl_action:
	push rbx

	cmp byte[rax+FLAGS], HD		; ctrl hidden & disable
	je .end
	cmp byte[rax+FLAGS], VD		; ctrl visible & disable
	je .end
	mov rbx, [rax+PTPOINTER]
	cmp byte[rbx+FLAGS], VD		; tile visible & disable
	je .end

	;---

	cmp byte[rax+TYPE], CHECK
	je .check
	cmp byte[rax+TYPE], LABEL	; (always disable)
	je .label
	cmp byte[rax+TYPE], ACTIVELABEL
	je .activelabel
	cmp byte[rax+TYPE], ARRAYALABEL
	je .arrayalabel
	cmp byte[rax+TYPE], INPUTSTRING
	je .inputstring
	cmp byte[rax+TYPE], BITMAP
	je .bitmap
	cmp byte[rax+TYPE], CUSTOMCONTROL
	je .customcontrol

	jmp .end

;---------------------------------------
;	CHECK
;---------------------------------------
.check:
	call check_action
	jmp .end
;---------------------------------------
;	LABEL
;---------------------------------------
.label:
	call label_action
	jmp .end
;---------------------------------------
;	ACTIVELABEL
;---------------------------------------
.activelabel:
	call activelabel_action
	jmp .end
;---------------------------------------
;	ARRAYALABEL
;---------------------------------------
.arrayalabel:
	call arrayalabel_action
	jmp .end
;---------------------------------------
;	INPUTSTRING
;---------------------------------------
.inputstring:
	call inputstring_action
	jmp .end
;---------------------------------------
;	BITMAP
;---------------------------------------
.bitmap:
	call bitmap_action
	jmp .end
;---------------------------------------
;	CUSTOMCONTROL
;---------------------------------------
.customcontrol:
	;push rbx
	mov rbx, qword[rdx+ACTION] 	; pointer to action user function
	call rbx
	;pop rbx
	jmp .end


.end:
	pop rbx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; ctrl_function -- Call the user function linked to the control
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
ctrl_function:
	push rbx
	push rdx

	mov rdx, rax

	mov rbx, qword[rdx+FUNCTION] 	; pointer to function
	cmp rbx, 0
	je .end

	call rbx			; call the function
	;jmp .end
.end:
	pop rdx
	pop rbx
	ret
; -----------------------------------------------------------------------------



; -----------------------------------------------------------------------------
;
; The draw and action functions for every type of control
;
; -----------------------------------------------------------------------------

;---------------------------------------
;	CHECK
;---------------------------------------
;---------------------------------------
; check_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
check_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .check_tohidedisable
	;---
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .check_tohidedisable
	;---

	call _tilectrl_draw_focus	; draw focus
	call _ctrl_draw_firstline	; draw first line

;.checkcontinue:
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add ah, 3
	call os_move_cursor

	mov rax, qword[rdx+VALUE] 	;
	mov cl, byte[rax]
	cmp cl, 0
	je .end
	mov al, 'X'
	mov rbx, qword[rdx+COLOR1] 	; *color text
	call os_print_char_with_color
	jmp .end			; end draw

.check_tohidedisable:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; check_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
check_action:
	push rax
	push rbx
	push rdx

	mov rdx, rax
.cicle:
	call os_input_key_wait

	cmp ah, 28			; If Enter key pressed, finish
	je .end

	cmp ah, 57			; if no Space key pressed, re-cicle
	jne .cicle

	; if Space key pressed, change value
	xor bx, bx
	mov rax, qword[rdx+VALUE] 	; pointer to value
	mov bl, byte[rax]		; value (0 or 1)
	btc bx, 0			; complement bit 0 (0->1, 1->0)
	mov byte[rax], bl		; store new value

	mov rax, rdx
	call os_ctrl_draw		; draw the control

	jmp .cicle

.end:
	pop rdx
	pop rbx
	pop rax
	ret
;---------------------------------------


;---------------------------------------
;	LABEL
;---------------------------------------
;---------------------------------------
; label_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
label_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .label_tohide		; (always disable)
	;---

	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	call os_move_cursor

	call _ctrl_draw_firstline	; draw first line
	je .end				; end draw

.label_tohide:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; label_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved	
label_action:
	ret
;---------------------------------------


;---------------------------------------
;	ACTIVELABEL
;---------------------------------------
;---------------------------------------
; activelabel_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
activelabel_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .alabel_tohidedisable
	;---
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .alabel_tohidedisable
	;---

	call _tilectrl_draw_focus	; draw focus
	call _ctrl_draw_firstline	; draw first line
	jmp .end			; end draw

.alabel_tohidedisable:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; activelabel_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
activelabel_action:
	push rax
	push rdx

	mov rdx, rax

	; reset TAG value
	mov qword[rdx+TAG], 0	 	; TAG value: 0=no actived

	call os_input_key_wait

	cmp ah, 28			; If no Enter key pressed, finish
	jne .end

	; if Enter key pressed, change TAG value
	mov qword[rdx+TAG], 1	 	; TAG value: 1=actived

.end:
	pop rdx
	pop rax
	ret
;---------------------------------------


;---------------------------------------
;	ARRAYALABEL
;---------------------------------------
;---------------------------------------
; arrayalabel_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
arrayalabel_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .aalabel_tohidedisable
	;---
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .aalabel_tohidedisable
	;---

	call _tilectrl_draw_focus	; draw focus
	call _ctrl_draw_firstline	; draw first line

;.aalabelcontinue:
	mov rax, qword[rdx+MLIST] 	; list pointer (array)
	mov rcx, qword[rdx+SELECTED]	; selected view (item)
.aalabelsw:
	cmp rcx, 0
	je .aalabelsw2
	add rax, qword[rdx+NUMBER]	; number of chars (add in array)
	inc rax				; null
	loop .aalabelsw
.aalabelsw2:
	mov rsi, rax			; list pointer (array)
	mov rbx, qword[rdx+COLOR3] 	; *color items
	mov rcx, 4			; print 4 items
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add ah, 2			; +2 to right

.aalabelprs:
	add al, 1			; next line
	call os_move_cursor

	; reverse color
	cmp rcx, 4
	jne .altr
	ror rbx, 32
	.altr:
	;--------------

	call os_print_string_with_color
	add rsi, qword[rdx+NUMBER]	; number of chars (add in array)
	inc rsi				; null

	; reverse color
	cmp rcx, 4
	jne .altl
	rol rbx, 32
	.altl:
	;--------------
	
	loop .aalabelprs		; next

	jmp .end			; end draw
	;--------------

.aalabel_tohidedisable:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; arrayalabel_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
arrayalabel_action:
	push rax
	push rbx
	push rdx

	mov rdx, rax

	; reset TAG value
	mov qword[rdx+TAG], 0	 	; TAG value: 0=no actived
	mov rbx, qword[rdx+SELECTED]	; save value

.more:
	call os_input_key_wait

	cmp ah, 72			; Up
	je .up
	cmp ah, 80			; Down
	je .down
	cmp ah, 28			; Enter
	je .enter

	jmp .other			; Other key

.up:
	cmp qword[rdx+SELECTED], 0
	je .more

	dec qword[rdx+SELECTED]		; selected view (item)
	mov rax, rdx
	call arrayalabel_draw
	jmp .more
.down:
	mov rax, qword[rdx+SELECTED]	; 0..(n-1)
	inc rax
	cmp rax, qword[rdx+NITEMS]
	je .more

	inc qword[rdx+SELECTED]		; selected view (item)
	mov rax, rdx
	call arrayalabel_draw
	jmp .more

.enter:
	; if Enter key pressed, change TAG value
	mov qword[rdx+TAG], 1		; TAG value: 1=actived
	jmp .end

.other:
	mov qword[rdx+SELECTED], rbx
	mov rax, rdx
	call arrayalabel_draw
	;jmp .end

.end:
	pop rdx
	pop rbx
	pop rax
	ret
;---------------------------------------


;---------------------------------------
;	INPUTSTRING
;---------------------------------------
;---------------------------------------
; inputstring_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
inputstring_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .inputstring_tohidedisable
	;---
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .inputstring_tohidedisable
	;---

	call _tilectrl_draw_focus	; draw focus
	call _ctrl_draw_firstline	; draw first line


	mov ah, byte[rdx+YSIZE]		; y size
	cmp ah, 1
	je .inputlinecontinue		; one line
	jmp .inputnotecontinue		; multi line

;------------------
.inputlinecontinue:
	mov rsi, qword[rdx+TEXT]
	call os_string_length		; rcx

	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add ah, 2
	add ah, cl
	call os_move_cursor

	mov rsi, qword[rdx+VALUE] 	; value (pointer to string)
	mov rbx, qword[rdx+COLOR3] 	; *color input (item)
	call os_print_string_with_color

	jmp .end			; end draw
;------------------

;------------------
.inputnotecontinue:
	push rdx
	mov rbx, qword[rdx+COLOR3] 	; *color input (item)
	mov al, byte[rdx+YPOS]		; y
	inc al				; skip first line
	mov ah, byte[rdx+XPOS] 		; x
	mov cl, byte[rdx+XSIZE]		; x sixe
	mov ch, byte[rdx+YSIZE]		; y size
	dec ch				; skip first line
	mov dl, ' '			; space
	call os_print_box_with_color
	pop rdx

	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add al, 1			; skip first line
	call os_move_cursor

	mov ecx, dword[colsrows_struct]	; save values

	mov [startrows], al		; new limits
	mov [startcols], ah		;
	add ah, byte[rdx+XSIZE]		;
	mov [ncols], ah			;

	mov rsi, qword[rdx+VALUE] 	; value (pointer to string)
	mov rbx, qword[rdx+COLOR3] 	; *color input (item)
	call os_print_string_with_color

	mov dword[colsrows_struct], ecx ; restore values

	jmp .end			; end draw
;------------------


.inputstring_tohidedisable:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; inputstring_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved	
inputstring_action:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rdx, rax

	mov ah, byte[rdx+YSIZE]		; y size
	cmp ah, 1
	je .inputlinecontinue		; one line
	jmp .inputnotecontinue		; multi line

;------------------
.inputlinecontinue:
	mov rsi, qword[rdx+TEXT]
	call os_string_length		; rcx
	mov rbx, rcx			; save value

	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add ah, 2
	add ah, cl
	call os_move_cursor

	call os_g_get_bgfg		; get current colors
	mov rsi, rcx			; save value
	mov rcx, qword[rdx+COLOR3] 	; *color input (item)
	call os_g_set_bgfg		; set new colors

	xor rcx, rcx
	mov cl, byte[rdx+XSIZE] 	; rcx = x size
	sub rcx, rbx			; -lenght of text
	sub rcx, 2 			; -focus and space
					; = max lenght of input
	mov rdi, qword[rdx+VALUE] 	; value (pointer to string)
	call os_input_string_with_cursor

	mov rcx, rsi
	call os_g_set_bgfg		; restore colors

	mov rax, rdx
	call os_ctrl_draw		; draw the control

	jmp .end
;------------------

;------------------
.inputnotecontinue:
	mov al, byte[rdx+YPOS]		; y
	mov ah, byte[rdx+XPOS] 		; x
	add al, 1			; skip first line
	call os_move_cursor

	mov ebx, dword[colsrows_struct]	; save values

	mov [startrows], al		; new limits
	mov [startcols], ah		;
	add ah, byte[rdx+XSIZE]		;
	mov [ncols], ah			;

	call os_g_get_bgfg		; get current colors
	mov rsi, rcx			; save value
	mov rcx, qword[rdx+COLOR3] 	; *color input (item)
	call os_g_set_bgfg		; set new colors

	xor rcx, rcx
	xor rax, rax
	mov al, byte[rdx+YSIZE]
	dec al
	mov cl, byte[rdx+XSIZE]
	mul cl				; rax = max lenght of input
	mov rcx, rax			; set rcx
	mov rdi, qword[rdx+VALUE] 	; value (pointer to string)
	call os_input_string_with_cursor

	mov dword[colsrows_struct], ebx ; restore values

	mov rcx, rsi
	call os_g_set_bgfg		; restore colors

	mov rax, rdx
	call os_ctrl_draw		; draw the control

	;jmp .end

.end:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret
;---------------------------------------


;---------------------------------------
;	BITMAP
;---------------------------------------
;---------------------------------------
; bitmap_draw
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved
bitmap_draw:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r12

	mov rdx, rax
	;---
	cmp byte[rdx+FLAGS], TH		; to hide
	je .bitmap_tohidedisable
	;---
	cmp byte[rdx+FLAGS], VD		; visible & disable
	je .bitmap_tohidedisable
	;---

	call _tilectrl_draw_focus	; draw focus
	call _ctrl_draw_firstline	; draw first line

	; clear area
	mov al, byte[rdx+YPOS]		; y start
	inc al
	mov ah, byte[rdx+XPOS] 		; x start
	mov ch, byte[rdx+YSIZE]
	mov cl, byte[rdx+XSIZE]
	mov rbx, qword[rdx+COLOR2] 	; *color control
	push rdx
	mov dl, ' '
	call os_print_box_with_color
	pop rdx

	mov rax, 2			; ysize font
	call os_g_get_fontinfo
	mov cl, byte[rdx+YSIZE]		;
	mul cl
	mov rsi, rax			; y size bitmap (pixel)
	mov rax, 3			; xsize font
	call os_g_get_fontinfo
	mov ch, byte[rdx+XSIZE]		;
	mul ch
	mov rdi, rax			; x size bitmap (pixel)

	mov al, byte[rdx+YPOS] 		; y start
	inc al
	mov ah, byte[rdx+XPOS] 		; x start
	call os_g_rowcol_to_yx

	mov rcx, qword[rdx+MLIST]	; pointer to image (in mem)
	call os_g_get_lfbregs		; set r14, r15
	mov r12, qword[rdx+NUMBER]	; zoom
	cmp r12, 0			; no zoom
	jne .draw
	inc r12				; (1:1)
.draw:
	call os_g_bitmap

	jmp .end			; end draw
	;--------------------


.bitmap_tohidedisable:
	call _ctrl_tohidedisable
	;jmp .end			; end draw

.end:
	pop r12
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
;---------------------------------------

;---------------------------------------
; bitmap_action
; IN :	RAX = pointer to control struct
; OUT:	All registers preserved	
bitmap_action:
	push rax
	push rdx

	mov rdx, rax
.more:
	xor rax, rax
	call os_input_key_wait

	cmp al, 0			; Special key?
	jne .more

	cmp ah, 78 			; +
	je .zoomm
	cmp ah, 74			; -
	je .zoomp
	cmp ah, 28			; Enter
	je .end

	jmp .more			; Other key

.zoomp:
	cmp qword[rdx+NUMBER], 0	; no zoom
	je .more
	cmp qword[rdx+NUMBER], 6	; max
	je .more
	inc qword[rdx+NUMBER]
	mov rax, rdx
	call bitmap_draw		; draw the control
	jmp .more
.zoomm:
	cmp qword[rdx+NUMBER], 0	; no zoom
	je .more
	cmp qword[rdx+NUMBER], 1	; min
	je .more
	dec qword[rdx+NUMBER]
	mov rax, rdx
	call bitmap_draw		; draw the control
	jmp .more
.end:
	pop rdx
	pop rax
	ret
;---------------------------------------


; =============================================================================
; EOF


