;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
; Graph Mode Functions
;-----------------------------------------------------------------------------------

;os_g_rowcol_to_yx
;os_g_yx_to_rowcol
;os_g_set_bgfg
;os_g_get_bgfg
;os_g_get_fontinfo
;os_g_get_lfbregs
;os_g_get_yxresolution
;os_g_is_graph
;os_g_pixel
;os_g_fillrect
;os_g_saverect
;os_g_restorerect
;os_g_line
;os_g_print
;os_g_bitmap
;os_g_screen_to_bitmap

; -----------------------------------------------------------------------------
; os_g_rowcol_to_yx -- Convert row/column in pixel coordinates
; IN:	AL  = row
;	AH  = column
; OUT:	RAX = y start	(pixel)
;	RBX = x start	(pixel)
os_g_rowcol_to_yx:
	push rcx
	push rdx

	xor rdx, rdx
	mov cx, ax
	mov rax, [os_System_Font_xsize]
	mul ch
	mov rbx, rax

	xor rdx, rdx
	mov rax, [os_System_Font_ysize]
	mul cl

	pop rdx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_yx_to_rowcol -- Convert pixel coordinates in row/column
; IN:	RAX = y start	(pixel)
;	RBX = x start	(pixel)
; OUT:	AL  = row
;	AH  = column
os_g_yx_to_rowcol:
	push rcx
	push rdx

	xor rdx, rdx
	mov rcx, [os_System_Font_ysize]
	div rcx
	xchg rax, rbx			; rax=>x, rbx=>row

	xor rdx, rdx
	mov rcx, [os_System_Font_xsize]
	div rcx
	mov ah, al

	mov al, bl

	pop rdx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_set_bgfg -- Set background-foreground colors
; IN :	RCX = color
; OUT:	All registers preserved
os_g_set_bgfg:

	mov [os_Color_bgfg], rcx

ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_bgfg -- Get background-foreground colors
; IN :	None
; OUT:	RCX = color
os_g_get_bgfg:

	mov rcx, [os_Color_bgfg]

ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_fontinfo -- Get font information
; IN :	RAX = requested info (1=font(address), 2=ysize, 3=xsize, 4=(x*y)size, 5=dispy, 6=dispx)
; OUT:  RAX = info
os_g_get_fontinfo:
	
	cmp rax, 1
	je .font
	cmp rax, 2
	je .font_y
	cmp rax, 3
	je .font_x
	cmp rax, 4
	je .font_xy
	cmp rax, 5
	je .font_dispy
	cmp rax, 6
	je .font_dispx
.font:
	mov rax, [os_System_Font]
	jmp .end
.font_y:
	mov rax, [os_System_Font_ysize]
	jmp .end
.font_x:
	mov rax, [os_System_Font_xsize]
	jmp .end
.font_xy:
	mov rax, [os_System_Font_xysize]
	jmp .end
.font_dispy:
	mov rax, [os_Disp_y]
	jmp .end
.font_dispx:
	mov rax, [os_Disp_x]

.end:

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_lfbregs -- Get LFB info in R14 an R15 registers
; IN :  None
; OUT:	R14 = pointer to LFB
;	R15 = bytes per line
os_g_get_lfbregs:

	xor r14, r14
	mov r14d, [vbe.LFB]
	xor r15, r15
	mov r15w, [vbe.bytesperline]

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_get_yxresolution -- Get screen resolution in pixel
; IN :  None
; OUT:	RAX = y resolution
;	RBX = x resolution
os_g_get_yxresolution:

	xor rax, rax
	xor rbx, rbx
	mov ax, [vbe.yres]
	mov bx, [vbe.xres]

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_is_graph -- Check if graph mode is on
; IN :  None
; OUT:	set carry (if carry is set = graph mode)
os_g_is_graph:

	cmp [os_GUI], 0
	jne .graph
	
	clc
	jmp .end
.graph:
	stc
.end:
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_pixel -- Draw a pixel on screen
; IN:	ECX = color			(ex.: 0x0000FF00 - argb)
; 	RAX = y
;	RBX = x
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_pixel:
	push rax
	push rbx
	push rdx

	cmp ax, [vbe.yres]
	jae .end
	cmp bx, [vbe.xres]
	jae .end

	; rax = x*4 + y*bytesxline (offset)
	shl rbx, 2
	mul r15
	add rax, rbx

	mov dword[r14+rax], ecx
.end:
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_fillrect -- Fill a rectangle
; IN:	ECX = color			(ex.: 0x0000FF00 - argb)
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_fillrect:
	push rax
	push rbx
	push rdx
	push rsi
	push r8

	mov r8, rbx			; save value
.rect:	
	mov rdx, rdi			; num. pixel x line
.line:
	call os_g_pixel			; *
	inc rbx				; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8			; restore value
	inc rax				; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_saverect -- Save a rectangle area of screen in memory
; IN:	RCX = memory location
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_saverect:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push r8

	mov r8, rbx			; save value
.rect:	
	mov rdx, rdi			; num. pixel x line
.line:
	push rax
	push rbx
	push rdx
	shl rbx, 2
	mul r15
	add rax, rbx
	mov ebx, dword[r14+rax]
	mov dword[rcx], ebx
	add rcx, 4
	pop rdx
	pop rbx
	pop rax

	inc rbx				; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8			; restore value
	inc rax				; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_restorerect -- Copy a rectangle area from memory to screen
; IN:	RCX = memory location
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_restorerect:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push r8

	mov r8, rbx			; save value
.rect:	
	mov rdx, rdi			; num. pixel x line
.line:
	push rax
	push rbx
	push rdx
	shl rbx, 2
	mul r15
	add rax, rbx
	mov ebx, dword[rcx]
	mov dword[r14+rax], ebx
	add rcx, 4
	pop rdx
	pop rbx
	pop rax

	inc rbx				; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8			; restore value
	inc rax				; y (next line)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_line - Draw a line using the Bresenham algorithm
; IN:	ECX = color		(ex.: 0x0000FF00 - argb)
;	RAX = y1
;	RBX = x1
;	RSI = y2
; 	RDI = x2
; OUT:	All registers preserved
os_g_line:				; Bresenham algorithm
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8 			; |x1-x2|
	push r9 			; |y1-y2|
	push r10
	push r11			; error
	push r12			; direction bit 0-> x, bit 1->y

	; clear error and direction
	mov r11, 0
	mov r12, 0

	; dxx=|x1-x2|
	mov r10, rbx
	sub r10, rdi 			; r10 = x1-x2
	; in r10 number without sign
	mov rdx, r10 			; save r10 in rdx
	neg r10 			; invert number sign
	cmp r10, rdx			; if r10<rdx then...
	jge .r1m
	mov r10, rdx			; ...place rdx in r10
	or r12b, 0x00000001		; set x direction bit 0
	.r1m:
	mov r8, r10 			; save |x1-x2| in r8

	; dyy=|y1-y2|
	mov r10, rax
	sub r10, rsi 			; r10 = y1-y2
	; in r10 number without sign
	mov rdx, r10 			; save r10 in rdx
	neg r10 			; invert number sign
	cmp r10, rdx			; if r10<rdx then...
	jge .r2m
	mov r10, rdx			; ...place rdx in r10
	or r12b, 0x00000010		; set y direction bit 1
	.r2m:
	mov r9, r10 			; save |y1-y2| in r9

	; select painting direction
	cmp r8, r10 			; |x1-x2|?|y1-y2|
	jge .h_line 			; if |x1-x2|>=|y1-y2|
	jmp .v_line 			; if |y1-y2|>|x1-x2|

;---------------------------------------
.h_line:
   	; painting always from left to right
	cmp rbx, rdi
	jg .swap_h			; if x1>x2 then interchange x and y coordinates
	jmp .start_paint_h
.swap_h: 				; x=y y=x
	xchg rax, rsi
	xchg rbx, rdi
	btc r12, 1			; reverse direction

.start_paint_h:
	mov r10, r8 			; r10=|x1-x2|  (counter)

	; start pixel y=y0 x=x0
	call os_g_pixel			; *

.loop_paint_h:
	inc rbx 			; inc x
	call os_g_pixel			; *

	mov rdx, r9
	add rdx, r11 			; error=error+|y1-y2|
	mov r11, rdx
	imul rdx, 00000002h 		; rdx=error*2
	cmp rdx, r8 			; if error*2>=|x1-x2|
	jge .inc_y
	jmp .next_step_h

.inc_y:
	test r12b, 0x00000010
	jz .incy
	dec rax				; dec y
	jmp .errh
.incy:
	inc rax 			; inc y
.errh:
	mov rdx, r11
	sub rdx, r8
	mov r11, rdx 			; error=error-|x1-x2|

.next_step_h: 
	dec r10				; decrement counter
	cmp r10, 0
	jne .loop_paint_h		; print next point
	jmp .end

;---------------------------------------
.v_line:
	; painting always from left to right
	cmp rax, rsi
	jg .swap_v			; if y1>y2 then interchange  x and y coordinates
	jmp .start_paint_v
.swap_v: 				; x=y y=x
	xchg rax, rsi
	xchg rbx, rdi
	btc r12, 0			; reverse direction

.start_paint_v:
	mov r10, r9 			; r10=|y1-y2|  (counter)

	; start pixel y=y0 x=x0
	call os_g_pixel			; *

.loop_paint_v:
	inc rax 			; inc y
	call os_g_pixel			; *

	mov rdx, r8
	add rdx, r11 			; error=error+|x1-x2|
	mov r11, rdx
	imul rdx, 00000002h 		; rdx=error*2
	cmp rdx, r9 			; if error*2>=|y1-y2|
	jge .inc_x
	jmp .next_step_v

.inc_x:
	test r12b, 0x00000001
	jz .incx
	dec rbx				; dec x
	jmp .errv
.incx:
	inc rbx 			; inc x
.errv:
	mov rdx, r11
	sub rdx, r9
	mov r11, rdx 			; error=error-|y1-y2|

.next_step_v:	
	dec r10				; decrement counter
	cmp r10, 0
	jne .loop_paint_v		; print next point

.end:
	pop r12
	pop r11
	pop r10
	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_print -- Print a bitmap char using a bytes mask
; IN:	RCX = color	(Background argb - Foreground argb)
;	RAX = y start	(pixel)
;	RBX = x start	"
;	RSI = y size	"
; 	RDI = x size	"
;	R8  = address of bytes mask
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_print:
	push rax
	push rbx
	push rdx
	push rdi
	push r8
	push r14

	; Displacement
	add rax, [os_Disp_y]
	add rbx, [os_Disp_x]

	; start point
	; rax = x*4 + y*bytesxline (offset)
	shl rbx, 2
	mul r15
	add rax, rbx
	add r14, rax			; add offset

.rect:
	;--------------------------------------------------------
	mov rbx, rdi
	mov rdx, 0
.line:
	;--------------- bytes mask
	mov al, byte[r8]
	inc r8
	cmp al, 'o'
	jne .back
	;---------------
.fore:
	mov dword[r14+rdx], ecx		; argb
	jmp .next
.back:
	ror rcx, 32
	mov dword[r14+rdx], ecx		; argb
	rol rcx, 32
.next:
	add rdx, 4
	dec rbx
	cmp rbx, 0
	jne .line
	;--------------------------------------------------------

	add r14, r15			; start of next line in LFB
	dec rsi				; dec counter of lines
	cmp rsi, 0			; until last line to draw
	jne .rect 			; next line

	pop r14
	pop r8
	pop rdi
	pop rdx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_g_bitmap -- Paint a bitmap image (with + or - zoom)
; IN:	RCX = address of image
; 	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
;	R12 = zoom (0 = no zoom)
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
; n.b.  x+xsize, y+ysize is the available area 
;	for painting the bitmap
os_g_bitmap:

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r10
	push r11
	push r14

	mov dx, word [rcx+00h]		; signature
	cmp dx, 19778			; = 'BM' (0x4d42)
	jne .end

	mov dx, word [rcx+1Ch]		; bpp
	cmp dx, 32			; 32 bit
	jne .end

	mov edx, dword [rcx+1Eh]	; compression 0=none, 1=RLE8, 2=RLE16
	cmp edx, 0
	jne .end

	mov r10, rcx
	xor rdx, rdx
	mov edx, dword [rcx+0Ah]	; offset of bitmap
	add r10, rdx			; now r10 points to start of image data

	xor rdx, rdx
	mov edx, dword [rcx+22h]	; size of image (bytes)
	add r10, rdx 			; now r10 points to end of image data

	xor rdx, rdx
	mov edx, dword [rcx+12h]	; X width (pixel)

	mov r11, rdx 			; X width
	shl r11, 2			; mul 4 (32bit)
	sub r10, r11			; now r10 points to start of last line of image data

	cmp rdi, rdx			; if rdi<=rdx then rdi not changed
	jbe .not1
	mov rdi, rdx			; if rdi>rdx then place rdx in rdi
	.not1:

	xor rdx, rdx
	mov edx, dword [rcx+16h]	; Y width (pixel)

	cmp rsi, rdx			; if rsi<=rdx then rsi not changed
	jbe .not2
	mov rsi, rdx			; if rsi>rdx then place rdx in rsi
	.not2:

	; start point
	; rax = x*4 + y*bytesxline (offset)
	shl rbx, 2
	mul r15
	add rax, rbx
	add r14, rax			; add offset

	.rect:
	;--------------------------------------------------------
	mov rbx, rdi
	mov rdx, 0
	.line:					
	mov ecx, dword[r10]		; data
	mov dword[r14+rdx], ecx		; argb
	add rdx, 4			; next pixel in LFB
;-----
	mov rcx, r12
.zoom1:
	add r10, 4			; next data
	dec rbx				; dec counter of pixels (x size)
	cmp rbx, 0
	je .next
	loop .zoom1
;-----
	jmp .line
	;--------------------------------------------------------

.next:
	mov rbx, rdi			; x size of available area
	shl rbx, 2			; mul 4 (32bit)
	sub r10, rbx			; return to start of line
	add r14, r15			; start of next line in LFB
;-----
	mov rcx, r12
.zoom2:
	sub r10, r11			; now r10 points to start of next line to paint
	dec rsi				; dec counter of lines (y size)
	cmp rsi, 0
	je .end
	loop .zoom2
;-----
	jmp .rect 			; next line

.end:
	pop r14
	pop r11
	pop r10
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------


; -------------------------------------------------------------------------------------
; os_g_screen_to_bitmap -- Save a rectangle area of screen in memory with bitmap format
; IN:	RCX = memory location
;	RAX = y start
;	RBX = x start
;	RSI = y size
; 	RDI = x size
; 	R14 = pointer to LFB
; 	R15 = bytes per line
; OUT:	All registers preserved
os_g_screen_to_bitmap:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push r8

	; bitmap header (54 bytes)
	;-------------------------
	push rax
	push rdx
	mov word[rcx+0], 'BM'	; 'BM' ID bitmap
	mov rax, rsi
	mul rdi
	shl rax, 2
	mov dword[rcx+34], eax	; Size of bitmap in bytes
	add rax, 54
	mov dword[rcx+ 2], eax	; Filesize of whole file (bytes)
	mov dword[rcx+ 6], 0	; Reserved
	mov dword[rcx+10], 54	; Offset of bitmap in file (BOF)
	mov dword[rcx+14], 40	; Length of BitmapInfoHeader
	mov dword[rcx+18], edi	; Horizontal width of bitmap in pixel
	mov dword[rcx+22], esi	; Vertical width of bitmap in pixel
	mov word[rcx+26], 1	; Number of planes
	mov word[rcx+28], 32	; Bits per pixel (BPP)
	mov dword[rcx+30], 0	; Compression Type
	mov dword[rcx+38], 0	; Horizontal resolution
	mov dword[rcx+42], 0	; Vertical resolution
	mov dword[rcx+46], 0	; Number of used colors
	mov dword[rcx+50], 0	; Number of important colors
	pop rdx
	pop rax

	add rcx, 54			; now points to image data
	;-----------------------

	add rax, rsi
	dec rax				; now points to last line

	mov r8, rbx			; save value
.rect:	
	mov rdx, rdi			; x size (num. pixel x line)
.line:
	push rax
	push rbx
	push rdx
	shl rbx, 2
	mul r15
	add rax, rbx
	mov ebx, dword[r14+rax]
	mov dword[rcx], ebx
	add rcx, 4
	pop rdx
	pop rbx
	pop rax

	inc rbx				; x (next pixel)
	dec rdx
	cmp rdx, 0
	jne .line

	mov rbx, r8			; restore value
	dec rax				; y (next line, in reverse order)
	dec rsi
	cmp rsi, 0
	jne .rect

.end:
	pop r8
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret
; -----------------------------------------------------------------------------



; ===================================================================================================
; EOF


;----------------------------------------------------------------------------------------------------
;
; Bitmap format:
;
; Offset	Size (Bytes)	Description
;
; 0000h	( 0)	2		'BM' ID bitmap
; 0002h	( 2)	4		Filesize of whole file
; 0006h	( 6)	4		Reserved
; 000Ah	(10)	4		Offset of bitmap in file (BOF)
; 000Eh	(14)	4		Length of BitmapInfoHeader (40 in Windows 3.x format, BIH starts after this header)
; 0012h	(18)	4		Horizontal width of bitmap in pixel
; 0016h	(22)	4		Vertical width of bitmap in pixel
; 001Ah	(26)	2		Number of planes
; 001Ch	(28)	2		Bits per pixel (BPP)
; 001Eh	(30)	4		Compression Type ( 0 = None, 1 = RLE8, 2 = RLE4 )
; 0022h	(34)	4		Size of bitmap in bytes
; 0026h	(38)	4		Horizontal resolution (Usually = 0)
; 002Ah	(42)	4		Vertical resolution (Usually = 0)
; 002Eh	(46)	4		Number of used colors (If = 0, number of used colors = BPP)
; 0032h	(50)	4		Number of important colors ( 0 = all colors are important )
; 0036h	(54)	?		Color table (BGRa) - Not present in 24 or 32 BPP
; BOF		?		Image data
;
; N.B. BMP files are stored in reverse order, from last line of data to first line.

