;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.4 - GPL3 - Apr 2013
;
;-----------------------------------------------------------------------------------
; Screen Output Functions
;-----------------------------------------------------------------------------------

;os_move_cursor
;os_inc_cursor
;os_dec_cursor
;os_inc_ycursor
;os_print_newline
;os_print_string
;os_print_string_with_color
;os_print_char
;os_print_char_with_color
;os_print_extrachar
;os_print_chars
;os_print_chars_with_color
;os_print_char_hex
;os_print_char_hex_with_color
;os_print_box_with_color
;os_screen_clear
;os_screen_clear_from_cursor
;os_row_clear
;os_hide_cursor
;os_show_cursor


; *****************************************************************************
; -----------------------------------------------------------------------------
; os_move_cursor -- Moves cursor
;  IN:	AH  = Column
;	AL  = Row
; OUT:	All registers preserved
os_move_cursor:
	push rdx
	push rcx
	push rbx
	push rax

	; Check limits
	cmp al, [startrows]
	jae .c1
	mov al, [startrows]
	.c1:

	cmp ah, [startcols]
	jae .c2
	mov ah, [startcols]
	.c2:

	cmp al, [nrows]
	jb .c3			; n-1
	mov al, [nrows]
	dec al
	.c3:

	cmp ah, [ncols]
	jb .c4			; n-1
	mov ah, [ncols]
	dec ah
	.c4:
	;--------------

	cmp [os_GUI], 0
	je .text

; --- GRAPH MODE Start ---

	mov [screen_cursor_x], ah
	mov [screen_cursor_y], al

	jmp .end

; --- GRAPH MODE End ---


; --- TEXT MODE Start ---
.text:
	mov [screen_cursor_x], ah
	mov [screen_cursor_y], al

	push rax
	and rax, 0x000000000000FFFF	; only keep the low 16 bits
	;calculate the new offset
	mov cl, 80			; columns
	mul cl				; AX = AL * CL
	xor rbx, rbx
	mov bl, [screen_cursor_x]
	add ax, bx
	shl ax, 1			; multiply by 2
	add rax, 0x00000000000B8000
	mov [screen_cursor_offset], rax
	pop rax				
					; Move the hardware cursor
	mov bh, ah
	mov bl, al
	xor ax, ax
	mov al, 0x50			; 80
	mul bl				; bl * al = ax
	movzx bx, bh
	add bx, ax
	mov al, 0x0E
	mov ah, bh
	mov dx, 0x03D4
	out dx, ax
	inc ax
	mov ah, bl
	out dx, ax	

; --- TEXT MODE End ---

.end:
	pop rax
	pop rbx
	pop rcx
	pop rdx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_inc_cursor -- Increment the cursor by one
;  IN:	Nothing
; OUT:	All registers preserved
os_inc_cursor:
	push rax

	mov ah, [screen_cursor_x]	; grab the current cursor location values
	mov al, [screen_cursor_y]
	inc ah
	cmp ah, [ncols]			; columns
	jne .os_inc_cursor_done

	inc al
	cmp al, [nrows]			; rows
	je .os_inc_cursor_last
	mov ah, [startcols]		; first column
	jmp .os_inc_cursor_done

.os_inc_cursor_last:
	dec al				; we are on the last column of the last row (bottom right)
	dec ah

.os_inc_cursor_done:
	call os_move_cursor

	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_dec_cursor -- Decrement the cursor by one
;  IN:	Nothing
; OUT:	All registers preserved
os_dec_cursor:
	push rax

	mov ah, [screen_cursor_x]	; Get the current cursor location values
	mov al, [screen_cursor_y]
	cmp ah, [startcols]		; Check if the cursor in located on the first column...
	jne .os_dec_cursor_done		; OK
	cmp al, [startrows]		; ...and check if the cursor in located on the first row
	je .os_dec_cursor_end
	dec al				; Wrap the cursor back to the above row
	mov ah, [ncols]			; columns

.os_dec_cursor_done:
	dec ah
	call os_move_cursor
.os_dec_cursor_end:
	pop rax
	ret
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; os_inc_ycursor -- Increment the cursor by one in Y direction
;  IN:	Nothing
; OUT:	All registers preserved
os_inc_ycursor:
	push rax

	mov ah, [screen_cursor_x]	; grab the current cursor location values
	mov al, [screen_cursor_y]

	inc al				; inc ycursor

	cmp al, [nrows]			; rows
	je .os_inc_ycursor_end		; we are already on the last row

	call os_move_cursor		; ok, move the cursor

.os_inc_ycursor_end:
	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_newline -- Reset cursor to start of next line
;  IN:	Nothing
; OUT:	All registers preserved
os_print_newline:
	push rax

	mov ah, [startcols]		; Set the cursor x value to first column
	mov al, [screen_cursor_y]	; Grab the cursor y value

	inc al
	cmp al, [nrows]	
	je .os_print_newline_last
	jmp .os_print_newline_done

.os_print_newline_last:
	dec al				; y value of last line

.os_print_newline_done:
	call os_move_cursor		; Update the cursor

	pop rax
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_string -- Displays text
;  IN:	RSI = message location (zero-terminated string)
; OUT:	All registers preserved
os_print_string:

	push rsi
	push rbx

	mov rbx, [os_Color_bgfg]
	call os_print_string_with_color

	pop rbx
	pop rsi

	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_string_with_color -- Displays text with color
;  IN:	RSI = message location (zero-terminated string)
;	RBX = color
; OUT:	All registers preserved
os_print_string_with_color:
	push rsi
	push rax

	cld				; Clear the direction flag.. we want to increment through the string

.os_print_string_nextchar:
	lodsb				; Get char from string and store in AL
	cmp al, 0			; Strings are Zero terminated.
	je .os_print_string_done	; If char is Zero then it is the end of the string
	cmp al, 10			; Check if there was a newline character in the string
	je .os_print_string_lf		; If so then we print a new line
	cmp al, 13			; Check if there was a newline character in the string
	je .os_print_string_cr		; If so then we print a new line

	call os_print_char_with_color	; Print char
	jmp .os_print_string_nextchar

.os_print_string_lf:
	call os_print_newline
	jmp .os_print_string_nextchar

.os_print_string_cr:
	mov al, [rsi]
	cmp al, 10
	je .os_print_string_nextchar
	call os_print_newline
	jmp .os_print_string_nextchar

.os_print_string_done:

	pop rax
	pop rsi
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_char -- Displays a char
;  IN:	AL  = char to display
; OUT:	All registers preserved
os_print_char:
	push rbx

	mov rbx, [os_Color_bgfg]
	call os_print_char_with_color

	pop rbx
	ret
; -----------------------------------------------------------------------------


; *****************************************************************************
; -----------------------------------------------------------------------------
; os_print_char_with_color -- Displays a char with color
;  IN:	AL  = char to display
;	RBX = color
; OUT:	All registers preserved
os_print_char_with_color:

	cmp [os_GUI], 0
	je .text

; --- GRAPH MODE Start ---

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8
	push r9

	mov r9, rbx			; save value

	and rax, 0x00000000000000FF	; clear except last 8 bit
	mov rcx, [os_System_Font_xysize]
	mul rcx
	mov r8, [os_System_Font]
	add r8, rax			; r8  = address of bytes mask

	xor rax, rax
	mov al, [screen_cursor_x]	; Get the current cursor location x value
	mov rcx, [os_System_Font_xsize]
	mul rcx
	mov rbx, rax			; rbx = x start (pixel)
	
	xor rax, rax
	mov al, [screen_cursor_y]	; Get the current cursor location y value
	mov rcx, [os_System_Font_ysize]
	mul rcx
					; rax = y start (pixel)

	mov rcx, r9			; rcx = background-foreground color
	mov rsi, [os_System_Font_ysize]	; rsi = y size (pixel)
	mov rdi, [os_System_Font_xsize]	; rdi = x size (pixel)

	xor r14, r14
	mov r14d, [vbe.LFB]		; r14 = pointer to LFB


	xor r15, r15
	mov r15w, [vbe.bytesperline]	; r15 = bytes per line

	call os_g_print
	call os_inc_cursor

	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	jmp .end

; --- GRAPH MODE End ---


; --- TEXT MODE Start ---
.text:
	push rdi
	push rbx

	call set_color_text	; bl = color (text mode)

	mov rdi, [screen_cursor_offset]
	stosb			; Store the character to video memory
	xchg al, bl		; Swap AL and BL as stosb uses AL
	stosb			; Store the color attribute to video memory
	xchg al, bl		; Swap AL and BL back again
	call os_inc_cursor

	pop rbx
	pop rdi

; --- TEXT MODE End ---

.end:
	ret
; -----------------------------------------------------------------------------



; *****************************************************************************
; -----------------------------------------------------------------------------
; os_print_extrachar -- Displays a extrachar (graph mode only)
;  IN:	AX  = extrachar to display
;	RBX = direction of printing ( 0=orizontal, 1=vertical, otherwise=fixed )
; OUT:	All registers preserved
os_print_extrachar:

	cmp [os_GUI], 0
	je .end

; --- GRAPH MODE Start ---		; graph mode only

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi
	push r8
	push r9

	mov r9, rbx			; save value

	and rax, 0x000000000000FFFF	; clear except last 16 bit
	mov rcx, [os_System_Font_xysize]
	mul rcx
	mov r8, [os_System_Font]
	add r8, rax			; r8  = address of bytes mask

	xor rax, rax
	mov al, [screen_cursor_x]	; Get the current cursor location x value
	mov rcx, [os_System_Font_xsize]
	mul rcx
	mov rbx, rax			; rbx = x start (pixel)
	
	xor rax, rax
	mov al, [screen_cursor_y]	; Get the current cursor location y value
	mov rcx, [os_System_Font_ysize]
	mul rcx
					; rax = y start (pixel)

	mov rcx, [os_Color_bgfg]	; rcx = background-foreground color
	mov rsi, [os_System_Font_ysize]	; rsi = y size (pixel)
	mov rdi, [os_System_Font_xsize]	; rdi = x size (pixel)

	xor r14, r14
	mov r14d, [vbe.LFB]		; r14 = pointer to LFB
	xor r15, r15
	mov r15w, [vbe.bytesperline]	; r15 = bytes per line

	call os_g_print

	cmp r9, 0
	je .orizontal
	cmp r9, 1
	je .vertical
	; otherwise=fixed
	jmp .popregs

	.orizontal:
	call os_inc_cursor
	jmp .popregs
	.vertical:
	call os_inc_ycursor
	;jmp .popregs
	
	.popregs:

	pop r9
	pop r8
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

; --- GRAPH MODE End ---

.end:

ret

; -----------------------------------------------------------------------------
; os_print_chars -- Displays text
;  IN:	RSI = message location (A string, not zero-terminated)
;	RCX = number of chars to print
; OUT:	All registers preserved
os_print_chars:
	push rbx

	mov rbx, [os_Color_bgfg]
	call os_print_chars_with_color

	pop rbx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_chars_with_color -- Displays text with color
;  IN:	RSI = message location (A string, not zero-terminated)
;	RBX = color
;	RCX = number of chars to print
; OUT:	All registers preserved
os_print_chars_with_color:
	push rsi
	push rcx
	push rax

	cld				; Clear the direction flag.. we want to increment through the string

.os_print_chars_nextchar:
	jrcxz .os_print_chars_done
	sub rcx, 1
	lodsb				; Get char from string and store in AL
	cmp al, 10			; Check if there was a newline character in the string
	je .os_print_chars_lf		; If so then we print a new line
	cmp al, 13			; Check if there was a newline character in the string
	je .os_print_chars_cr		; If so then we print a new line

	call os_print_char_with_color	; Print char
	jmp .os_print_chars_nextchar

.os_print_chars_lf:
	call os_print_newline
	jmp .os_print_chars_nextchar

.os_print_chars_cr:
	mov al, [rsi]
	cmp al, 10
	je .os_print_chars_nextchar
	call os_print_newline
	jmp .os_print_chars_nextchar

.os_print_chars_done:

	pop rax
	pop rcx
	pop rsi
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_char_hex -- Displays a char in hex mode
;  IN:	AL  = char to display
; OUT:	All registers preserved
os_print_char_hex:
	push rbx
	push rax

	mov rbx, hextable

	push rax	; save rax for the next part
	shr al, 4	; we want to work on the high part so shift right by 4 bits
	xlatb
	call os_print_char

	pop rax
	and al, 0x0f	; we want to work on the low part so clear the high part
	xlatb
	call os_print_char

	pop rax
	pop rbx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_char_hex_with_color -- Displays a char in hex mode
;  IN:	AL  = char to display
;	RBX = color
; OUT:	All registers preserved
os_print_char_hex_with_color:
	push rcx
	push rbx
	push rax

	mov rcx, rbx
	mov rbx, hextable


	push rax	; save rax for the next part
	shr al, 4	; we want to work on the high part so shift right by 4 bits
	xlatb
	push rbx
	mov rbx, rcx
	call os_print_char_with_color
	pop rbx

	pop rax
	and al, 0x0f	; we want to work on the low part so clear the high part
	xlatb
	push rbx
	mov rbx, rcx
	call os_print_char_with_color
	pop rbx


	pop rax
	pop rbx
	pop rcx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_print_box_with_color -- Print a box with a char
; IN :	RBX = color
; 	AL  = Y
; 	AH  = X
; 	CL  = X size
; 	CH  = Y size
; 	DL  = char
; OUT:	All registers preserved
os_print_box_with_color:
	push rax
	push rcx	
	push rdi

	movzx di, ch			; save value
	and rcx, 0x00000000000000FF	; clear except last 8 bit
.more:
	call os_move_cursor
	
	push rax
	push rcx
	mov al, dl			; char
.line:
	call os_print_char_with_color
	loop .line
	pop rcx
	pop rax

	inc al				; next line
	dec di
	cmp di, 0			; finished?
	jne .more

	pop rdi
	pop rcx
	pop rax
	ret
; -----------------------------------------------------------------------------


; *****************************************************************************
; -----------------------------------------------------------------------------
; os_screen_clear -- Clear the screen
;  IN:	Nothing
; OUT:	All registers perserved
os_screen_clear:

	cmp [os_GUI], 0
	je .text

; --- GRAPH MODE Start ---

	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	mov rbx, [os_Color_bgfg]
	ror rbx, 32

	xor rdi, rdi
	mov edi, [vbe.LFB]
	xor rsi, rsi
	mov si, [vbe.bytesperline]
	xor rdx, rdx
	mov dx, [vbe.yres]

	.rect:
	;--------------------------------------------------------
	xor rcx, rcx
	mov cx, [vbe.xres]
	mov rax, 0
	.line:					; a   r   g   b
	mov dword[rdi+rax], ebx 		; 000 000 000 000
	add rax, 4
	loop .line
	;--------------------------------------------------------

	add rdi, rsi				; start of next line in LFB
	dec rdx
	cmp rdx, 0
	jne .rect 

	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax

	jmp .end

; --- GRAPH MODE End ---

; --- TEXT MODE Start ---
.text:
	push rdi
	push rcx
	push rax

	mov ax, 0x0720			; 0x07 for black background/white foreground, 0x20 for space (black) character
	mov rdi, 0x00000000000B8000	; Address for start of color video memory
	mov rcx, 2000			; 2000 words
	rep stosw			; Clear the screen. Store word in AX to RDI, RCX times
	

	pop rax
	pop rcx
	pop rdi

; --- TEXT MODE End ---

.end:
	ret
; -----------------------------------------------------------------------------

; *****************************************************************************
; -----------------------------------------------------------------------------
; os_screen_clear_from_cursor -- Clear the screen from cursor to end
;  IN:	Nothing
; OUT:	All registers preserved
os_screen_clear_from_cursor:

	cmp [os_GUI], 0
	je .text

; --- GRAPH MODE Start ---

	push rax
	push rcx

	mov al, [screen_cursor_y]	; Get the current cursor location y value
	mov ah, [screen_cursor_x]	; Get the current cursor location x value
	push rax			; Save values
	call os_row_clear

	mov cl, [nrows]
	dec cl				; nrows-1
.next:
	cmp al, cl			; if last
	je .ok

	mov ah, 0			; [startcols]
	inc al
	call os_row_clear
	jmp .next
.ok:
	pop rax				; Restore values
	call os_move_cursor		; of cursor location

	pop rcx
	pop rax

	jmp .end

; --- GRAPH MODE End ---

; --- TEXT MODE Start ---
.text:
	push rdi
	push rcx
	push rax

	mov ax, 0x0720			; 0x07 for black background/white foreground, 0x20 for space (black) character
	mov rdi, [screen_cursor_offset]	; Address for start of color video memory [0x00000000000B8000 + offset]
	mov rcx, 0x00000000000B8000
	add rcx, 4000-2			; 2000 words
	sub rcx, [screen_cursor_offset]			
	rep stosw			; Clear the screen. Store word in AX to RDI, RCX times
	

	pop rax
	pop rcx
	pop rdi

; --- TEXT MODE End ---

.end:
	ret
; -----------------------------------------------------------------------------

; *****************************************************************************
; -----------------------------------------------------------------------------
; os_row_clear -- Clear a row from a selected column.
;  IN:	AL  = row to clear
; 	AH  = column to start
; OUT:	All registers preserved
os_row_clear:
	push rdi
	push rsi
	push rcx
	push rbx
	push rax

	push rax
	call os_move_cursor

	xor rcx, rcx
	mov cl, [ncols]			; Num. space chars
	sub cl, ah

	cmp [os_GUI], 0
	je .text

; --- GRAPH MODE Start ---

	mov rbx, [os_Color_bgfg]
	mov al, ' '
.graph:
	call os_print_char_with_color
	loop .graph
	jmp .movecur

; --- GRAPH MODE End ---


; --- TEXT MODE Start ---

.text:
	mov rdi, [screen_cursor_offset]
	mov ax, 0x0720			; 0x07 for black background/white foreground, 0x20 for space (black) character
	rep stosw			; Write the character and attribute with one call

; --- TEXT MODE End ---

.movecur:
	pop rax
	call os_move_cursor

	pop rax
	pop rbx
	pop rcx
	pop rsi
	pop rdi
	ret
; -----------------------------------------------------------------------------

; --- TEXT MODE Start ---

; -----------------------------------------------------------------------------
; os_hide_cursor -- Turns off cursor in text mode
;  IN:	Nothing
; OUT:	All registers preserved
os_hide_cursor:
	push rdx
	push rbx
	push rax

	mov dx, 0x03d4
	mov ax, 0x000a		; Cursor Start Register
	out dx, ax
	inc dx
	xor ax, ax
	in al, dx
	mov bl, al
	or bl, 00100000b	; Bit 5 set to 1 to disable cursor
	dec dx
	mov ax, 0x000a		; Cursor Start Register
	out dx, ax
	inc dx
	mov al, bl
	out dx, al

	pop rax
	pop rbx
	pop rdx
	ret
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; os_show_cursor -- Turns on cursor in text mode
;  IN:	Nothing
; OUT:	All registers preserved
os_show_cursor:
	push rdx
	push rbx
	push rax

	mov dx, 0x03d4
	mov ax, 0x000a		; Cursor Start Register
	out dx, ax
	inc dx
	xor ax, ax
	in al, dx
	mov bl, al
	and bl, 11011111b	; Bit 5 set to 0 to enable cursor
	dec dx
	mov ax, 0x000a		; Cursor Start Register
	out dx, ax
	inc dx
	mov al, bl
	out dx, al

	pop rax
	pop rbx
	pop rdx
	ret
; -----------------------------------------------------------------------------




; =============================================================================
; EOF
